package org.diplomaUPV.app.scrapping;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.diplomaUPV.app.rest.dao.ModalidadDao;
import org.diplomaUPV.app.rest.entity.Evento;
import org.diplomaUPV.app.rest.entity.Modalidad;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.beans.factory.annotation.Autowired;


public class ScraperCalendarioCarrerasPopulares{

	private static String URL_CARRERAS_POPULARES = "http://www.carreraspopulares.com/"; 
	private static String URL_DETALLES_CARRERA = "http://www.carreraspopulares.com/v5-calendario/WR_KK01_menu_right.asp?fr_cl_edicion=";
	private int totalCCAA = 19;
	private int meses = 12;
	private Map<String, Modalidad> listadoModalidades;
	private List<Evento> listadoEventos = new ArrayList<Evento>();
	private static String MODALIDAD = "MODALIDAD: ";
	private static String POBLACION = "POBLACIÓN: ";
	private static String DISTANCIA = "DISTANCIA: ";
	private static String SALIDA = "LUGAR DE SALIDA: ";
	private static String MAS_INFO = "MÁS INFORMACIÓN: ";


	public List<Evento> extraerInfo(Map<String, Modalidad> listadoModalidades) throws IOException, ParseException {
		Document doc;
		String CCAACountString;
		this.listadoModalidades = listadoModalidades;
		
		for (int CCAACount = 1; CCAACount <= totalCCAA; CCAACount++) {
			CCAACountString = String.valueOf(CCAACount);
			doc = Jsoup.connect(URL_CARRERAS_POPULARES + "wspubli/service/ws_carreras.php")
					.data("func", "get_races")
					.data("idUser", "26432")
					.data("clave", "994508fde476e785920e50c1669ecb33")
					.data("in_tipo_cons", "1")
					.data("in_id_deporte", "1")
					.data("in_id_pais", "10")
					.data("in_id_comunidad", CCAACountString)
					.post();

			Elements infoCarreras = doc.getElementsByTag("carrera");
			for (Element infoCarrera : infoCarreras) {
				String idCodificadaCarrera = infoCarrera.attr("idcodificada");
				String fechaCarrera = infoCarrera.attr("fechaed");
				Date dateCarrera = extraerFecha(fechaCarrera);
				if(dateCarrera.before(new Date()))continue;
				Evento infoEvento = new Evento().withIdComunidadAutonoma(new Long(CCAACount)).withFecha(dateCarrera);

				StringBuilder urlCarrera = new StringBuilder(URL_DETALLES_CARRERA)
				.append(idCodificadaCarrera)
				.append("&fr_id_comunidad=")
				.append(CCAACountString);

				extraeInfoEvento(urlCarrera.toString(), infoEvento);				
				listadoEventos.add(infoEvento);
			}
		}
		return listadoEventos;
	}

	private void extraeInfoEvento(String linkEvento, Evento infoEvento) throws IOException, ParseException {		 
		Document doc = Jsoup.connect(linkEvento).get();
		Elements textoDetalles = doc.select("div.extended-info > p");
		for (Element textoDetalle : textoDetalles) {
			String textoDetalleString = textoDetalle.text().toUpperCase();
			int longitudTextoDetalle = textoDetalleString.length();
			if(textoDetalleString.startsWith(MODALIDAD)){
				String textoModalidad = textoDetalleString.substring(MODALIDAD.length(), longitudTextoDetalle);
				System.out.println("textoModalidad " + textoModalidad );
				System.out.println("listadoModalidades " + listadoModalidades.size() );
				Modalidad modalidadSeleccionada = listadoModalidades.get(textoModalidad.toUpperCase());
				if(modalidadSeleccionada != null){
					Long idModalidad = modalidadSeleccionada.getId();
					Long idDisciplina = modalidadSeleccionada.getIdDisciplina();
					infoEvento.setIdModalidad(idModalidad);
					infoEvento.setIdDisciplina(idDisciplina);
				}
			}
			if(textoDetalleString.startsWith(POBLACION)){
				String localidad = textoDetalleString.substring(POBLACION.length(), longitudTextoDetalle) ;
				infoEvento.setLocalidad(localidad);
				try {
					infoEvento.setLocalidadGmaps(Utils.getLatLongPositions(localidad));
				} catch (Exception e) {
					e.printStackTrace();
				}
			}

			if(textoDetalleString.startsWith(MAS_INFO)){
				String urlMasInfoString = textoDetalle.select("a").attr("href");
				infoEvento.setUrlEvento(URL_CARRERAS_POPULARES + urlMasInfoString);
			}
		}

		Elements tituloEventoElements = doc.select("div.extended-info > h3");
		for (Element tituloEvento : tituloEventoElements) {
			String tituloEventoString = tituloEvento.text();
			infoEvento.setTitulo(tituloEventoString);
			System.out.println("Parsing " + tituloEvento );
		}
		infoEvento.setFechaCreacion(new Date());
	}


	private Date extraerFecha(String stringFecha) throws ParseException {
		SimpleDateFormat formatoDeFecha = new SimpleDateFormat("dd/MM/yyyy");
		Date date = formatoDeFecha.parse(stringFecha);
		return date;
	}

}
