package com.diplomaAndroidUpv.org.competicionesDeportivas.Activity;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.DatePickerDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.provider.Settings;
import android.transition.Transition;
import android.view.View;
import android.view.ViewAnimationUtils;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.SeekBar;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.diplomaAndroidUpv.org.competicionesDeportivas.R;
import com.diplomaAndroidUpv.org.competicionesDeportivas.Util.CompetitionsFilter;
import com.diplomaAndroidUpv.org.competicionesDeportivas.Util.DialogCreator;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;


/**
 * Activity encargada de mostrar las distintas opciones de filtrado sobrel el listado de competiciones
 * disponibles
 */
public class FilterOptionsActivity extends Activity {

    private EditText fromDateFilter, toDateFilter;
    private DatePickerDialog fromDatePickerDialog, toDatePickerDialog;
    private Date fromDate, toDate;
    private CheckBox chkDate, chkCa, chkModality, chkPosition;
    private Spinner spinnerCa, spinnerModality;
    private SeekBar seekBarPosition;
    private TextView selValuePosition;
    private Button applyFiltersBtn;
    private static final long MIN_TIME = 10 * 1000;
    private static final long MIN_DISTANCE = 5;
    private LocationManager locationManager;
    private String provider;
    public static int FILTER_APPLIED = 1;
    public static String FILTER_ID = "MyFilter";
    private CompetitionsFilter filter;
    SimpleDateFormat dateFormatter = new SimpleDateFormat("dd/MM/yyyy");


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_filter_competitions);
        initLocationManager();
        initDateFilter();
        initSpinners();
        initSeekBar();
        initCheckBox();
        initFilterButton();
        getStoredFilter();
    }

    /**
     * Obtiene el valor del filtro previamente utilizado por el usuario
     * @return
     */
    public void getStoredFilter() {
        this.filter =  (CompetitionsFilter) getIntent().getSerializableExtra(FilterOptionsActivity.FILTER_ID);
        if(this.filter.getModalidad() != null){
            this.chkModality.setChecked(true);
            this.spinnerModality.setEnabled(chkModality.isChecked());
            this.spinnerModality.setClickable(chkModality.isChecked());
            this.spinnerModality.setSelection(this.filter.getModalidad().intValue());
        }
        if(this.filter.getcA() != null){
            this.chkCa.setChecked(true);
            this.spinnerCa.setEnabled(chkCa.isChecked());
            this.spinnerCa.setClickable(chkCa.isChecked());
            this.spinnerCa.setSelection(this.filter.getcA().intValue());
        }
        if(this.filter.getInitDate() != null || this.filter.getEndDate() != null ){
            this.chkDate.setChecked(true);
            fromDateFilter.setFocusable(true);
            fromDateFilter.setClickable(true);
            fromDateFilter.setEnabled(true);
            toDateFilter.setFocusable(true);
            toDateFilter.setClickable(true);
            toDateFilter.setEnabled(true);
            if(this.filter.getInitDate() != null){
                this.fromDateFilter.setText(dateFormatter.format(this.filter.getInitDate().getTime()));
            }
            if(this.filter.getEndDate() != null){
                this.toDateFilter.setText(dateFormatter.format( this.filter.getEndDate().getTime()));
            }
        }
        if(this.filter.getKmDistance() != null){
            this.chkPosition.setChecked(true);
            this.seekBarPosition.setEnabled(chkPosition.isChecked());
            this.seekBarPosition.setClickable(chkPosition.isChecked());
            this.seekBarPosition.setFocusable(chkPosition.isChecked());
            this.seekBarPosition.setProgress(this.filter.getKmDistance().intValue());
        }
    }
    /**
     * Inicializa el botón de filtrado disponible en pantalla. Una vez se realice click sobre este, se aplicarán los
     * filtros seleccionados sobre el listado de competiciones
     */
    private void initFilterButton() {
        applyFiltersBtn = (Button)findViewById(R.id.applyFiltersBtn);
        applyFiltersBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent intent = new Intent();
                intent.putExtra(FILTER_ID, getFilter());
                setResult(FILTER_APPLIED, intent);
                finish();
            }
        });
    }

    /**
     * Obtiene el objeto filter resultante
     * @return
     */
    public CompetitionsFilter getFilter() {
        filter = new CompetitionsFilter();
        if(chkPosition.isChecked()) {
            filter.setKmDistance(new Long(seekBarPosition.getProgress()));
        }
        if(chkCa.isChecked()) {
            filter.setcA(spinnerCa.getSelectedItemId());
        }
        if(chkModality.isChecked()) {
            filter.setModalidad(spinnerModality.getSelectedItemId());
        }
        if(chkDate.isChecked() && fromDate != null) {
            filter.setInitDate(fromDate);
        }
        if(chkDate.isChecked() && toDate != null) {
            filter.setEndDate(toDate);
        }
        return filter;
    }

    /**
     * Inicializa el filtro por posicion
     */
    private void initSeekBar() {
        selValuePosition = (TextView)findViewById(R.id.selValuePosition);
        seekBarPosition = (SeekBar)findViewById(R.id.seekBarPosition);
        seekBarPosition.setClickable(false);
        seekBarPosition.setEnabled(false);
        seekBarPosition.setFocusable(false);
        seekBarPosition.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int i, boolean b) {
                selValuePosition.setText(String.valueOf(seekBar.getProgress()) + "Km");
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {
            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {
            }
        });
    }

    /**
     * Inicializa los distintos combos de pantalla (selección de comunidad autónoma y modalidad)
     */
    private void initSpinners() {
        spinnerCa = (Spinner)findViewById(R.id.spinnerCa);
        spinnerCa.setEnabled(false);
        spinnerCa.setClickable(false);
        spinnerModality = (Spinner)findViewById(R.id.spinnerModality);
        spinnerModality.setEnabled(false);
        spinnerModality.setClickable(false);
    }

    /**
     * Inicializa el filtro por fecha
     */
    private void initDateFilter() {
        fromDateFilter = (EditText)findViewById(R.id.fromDateFilter);
        fromDateFilter.setClickable(false);
        fromDateFilter.setEnabled(false);
        fromDateFilter.setFocusable(false);
        toDateFilter = (EditText)findViewById(R.id.toDateFilter);
        toDateFilter.setClickable(false);
        toDateFilter.setEnabled(false);
        toDateFilter.setFocusable(false);
        fromDateFilter.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                InputMethodManager inputManager = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if (getCurrentFocus() != null && getCurrentFocus().getWindowToken() != null) {
                    inputManager.hideSoftInputFromWindow(getCurrentFocus().getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
                }
                Calendar newCalendar = Calendar.getInstance();
                fromDatePickerDialog = new DatePickerDialog(FilterOptionsActivity.this,
                        new DatePickerDialog.OnDateSetListener() {
                            @Override
                            public void onDateSet(DatePicker datePicker, int year, int monthOfYear, int dayOfMonth) {
                                Calendar newDate = Calendar.getInstance();
                                newDate.set(year, monthOfYear, dayOfMonth);
                                fromDateFilter.setText(dateFormatter.format(newDate.getTime()));
                                fromDate = newDate.getTime();
                                if(toDate != null && fromDate.compareTo(toDate) > 0){
                                    Toast.makeText(FilterOptionsActivity.this, R.string.dates_error, Toast.LENGTH_SHORT).show();
                                    toDateFilter.setText("");
                                    toDate = null;
                                }
                            }
                        },
                        newCalendar.get(Calendar.YEAR), newCalendar.get(Calendar.MONTH), newCalendar.get(Calendar.DAY_OF_MONTH));
                fromDatePickerDialog.show();
            }
        });
        toDateFilter.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                InputMethodManager inputManager = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if (getCurrentFocus() != null && getCurrentFocus().getWindowToken() != null) {
                    inputManager.hideSoftInputFromWindow(getCurrentFocus().getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
                }
                Calendar newCalendar = Calendar.getInstance();
                toDatePickerDialog = new DatePickerDialog(FilterOptionsActivity.this,
                        new DatePickerDialog.OnDateSetListener() {
                            @Override
                            public void onDateSet(DatePicker datePicker, int year, int monthOfYear, int dayOfMonth) {
                                Calendar newDate = Calendar.getInstance();
                                newDate.set(year, monthOfYear, dayOfMonth);
                                toDateFilter.setText(dateFormatter.format(newDate.getTime()));
                                toDate = newDate.getTime();

                                if(fromDate != null && fromDate.compareTo(toDate) > 0){
                                    Toast.makeText(FilterOptionsActivity.this, R.string.dates_error, Toast.LENGTH_SHORT).show();
                                    toDateFilter.setText("");
                                    toDate = null;
                                }
                            }
                        },
                        newCalendar.get(Calendar.YEAR), newCalendar.get(Calendar.MONTH), newCalendar.get(Calendar.DAY_OF_MONTH));
                toDatePickerDialog.show();
            }
        });
    }

    /**
     * Establece el comportamiento de activación y desactivación en función de la selección de los diferentes
     * checkbox de pantalla.
     */
    private void initCheckBox() {
        chkDate = (CheckBox)findViewById(R.id.chkDate);
        chkDate.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                fromDateFilter.setFocusable(chkDate.isChecked());
                fromDateFilter.setClickable(chkDate.isChecked());
                fromDateFilter.setEnabled(chkDate.isChecked());
                toDateFilter.setFocusable(chkDate.isChecked());
                toDateFilter.setClickable(chkDate.isChecked());
                toDateFilter.setEnabled(chkDate.isChecked());

                if (!chkDate.isChecked()) {
                    fromDate = null;
                    toDate = null;
                    fromDateFilter.setText(null);
                    toDateFilter.setText(null);
                }
            }
        });
        chkCa = (CheckBox)findViewById(R.id.chkCa);
        chkCa.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                spinnerCa.setEnabled(chkCa.isChecked());
                spinnerCa.setClickable(chkCa.isChecked());
                if (!chkCa.isChecked()) {
                    spinnerCa.setSelection(0);
                }
            }
        });
        chkModality = (CheckBox)findViewById(R.id.chkModality);
        chkModality.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                spinnerModality.setEnabled(chkModality.isChecked());
                spinnerModality.setClickable(chkModality.isChecked());
                if (!chkModality.isChecked()) {
                    spinnerModality.setSelection(0);
                }
            }
        });

        chkPosition = (CheckBox)findViewById(R.id.chkPosition);
        chkPosition.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                seekBarPosition.setEnabled(chkPosition.isChecked());
                seekBarPosition.setClickable(chkPosition.isChecked());
                seekBarPosition.setFocusable(chkPosition.isChecked());
                if(chkPosition.isChecked()){
                    getCurrentPosition();
                }else{
                    seekBarPosition.setProgress(0);
                }
            }
        });
    }

    /**
     * Inicializa elementos de posicionamiento geográfico necesarios para el filtrado por posición
     */
    private void initLocationManager() {
        locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);
        Criteria locationCritera = new Criteria();
        locationCritera.setAccuracy(Criteria.ACCURACY_FINE);
        locationCritera.setAltitudeRequired(false);
        locationCritera.setCostAllowed(false);
        provider = locationManager.getBestProvider(locationCritera, true);
    }


    /**
     * Obtiene la posición actual
     */
    private void getCurrentPosition() {
        String location =  PreferenceManager.getDefaultSharedPreferences(getApplicationContext()).getString("locationString", null);
        if (location == null) {
            DialogCreator.createDialogSavePositionActivation(FilterOptionsActivity.this).show();
            chkPosition.setChecked(false);
            seekBarPosition.setEnabled(false);
            seekBarPosition.setClickable(false);
            seekBarPosition.setFocusable(false);
        }
    }
}
