package com.diplomaAndroidUpv.org.competicionesDeportivas.Activity;

import android.app.SearchManager;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.util.SparseArray;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.animation.AccelerateInterpolator;
import android.view.animation.DecelerateInterpolator;
import android.view.inputmethod.InputMethodManager;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.SearchView;
import android.widget.Toast;

import com.diplomaAndroidUpv.org.competicionesDeportivas.Entity.Competition;
import com.diplomaAndroidUpv.org.competicionesDeportivas.Listener.HidingScrollListener;
import com.diplomaAndroidUpv.org.competicionesDeportivas.Listener.RecyclerItemClickListener;
import com.diplomaAndroidUpv.org.competicionesDeportivas.R;
import com.diplomaAndroidUpv.org.competicionesDeportivas.RecyclerView.Competitions.RecyclerAdapterCompetition;
import com.diplomaAndroidUpv.org.competicionesDeportivas.Util.CompetitionsFilter;
import com.diplomaAndroidUpv.org.competicionesDeportivas.Util.DialogCreator;
import com.diplomaAndroidUpv.org.competicionesDeportivas.Util.JsonParser;
import com.diplomaAndroidUpv.org.competicionesDeportivas.Util.Utils;
import com.getbase.floatingactionbutton.FloatingActionButton;
import com.getbase.floatingactionbutton.FloatingActionsMenu;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapFragment;
import com.google.android.gms.maps.model.BitmapDescriptor;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;
import com.roomorama.caldroid.CaldroidFragment;
import com.roomorama.caldroid.CaldroidListener;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;


/**
 * Activity encargade de mostrar el listado de competiciones disponibles. Dispone de opciones para modificar la forma en que se visualizan estas
 * asi como opciones de filtrado
 */
public class ListCompetitionsActivity extends AppCompatActivity {

    private FloatingActionButton listBtn, calendarBtn, mapBtn, filterBtn;
    private FloatingActionsMenu fabButtonMenu;
    private RecyclerView recyclerCompetitionsView;
    private List<Competition> filteredCompetitionsList, completeCompetitionsList;
    private CaldroidFragment caldroidFragment;
    private LinearLayout calendarCompetitionsView, mapsCompetitionsView;
    private MapFragment competitionsMapFragment;
    private GoogleMap competitionsMap;
    public static SparseArray<Bitmap> sPhotoCache = new SparseArray<Bitmap>(4);
    public static String JSON_COMPETICIONES_ID = "JSON_COMPETICIONES";
    JsonParser jsonParser = new JsonParser();
    private static final DateFormat FORMATTER = SimpleDateFormat.getDateInstance();
    private Long idDisciplina;
    private CompetitionsFilter filter;
    private ArrayList<Date> lastSelectedDates = new ArrayList<Date>();



    @Override
    protected void onCreate(Bundle savedInstanceState) {
        getWindow().requestFeature(Window.FEATURE_ACTION_BAR);
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_list_competitions);
        idDisciplina = getIntent().getLongExtra("idDisciplina", new Long(1));
        initFabButton();
        initRecyclerView();
        initCalendar();
        initMap();
        initFilterBtn();
        getCompetitionsList();
    }

    /**
     * Inicializa el layout oculto que muestra el listado de carreras posicionados sobre un mapa
     */
    private void initMap() {
        mapsCompetitionsView = (LinearLayout)findViewById(R.id.mapsCompetitionsView);
        competitionsMapFragment = (MapFragment) getFragmentManager().findFragmentById(R.id.competitionsMap);
        competitionsMap = competitionsMapFragment.getMap();

        double lat = Double.longBitsToDouble(PreferenceManager.getDefaultSharedPreferences(getApplicationContext()).getLong("myLat", 0));
        double lng = Double.longBitsToDouble(PreferenceManager.getDefaultSharedPreferences(getApplicationContext()).getLong("myLong",0));

        if(lat == 0){lat = 40.3852726; }
        if(lng == 0){lng =  -3.2798855; }

        float zoom = getIntent().getFloatExtra("zoom", 6.25f);
        LatLng position = new LatLng(lat, lng);
        competitionsMap.moveCamera(CameraUpdateFactory.newLatLngZoom(position, zoom));
    }

    /**
     * Inicializa el botón de filtrado del menu desplegable
     */
    private void initFilterBtn(){
        filterBtn = (FloatingActionButton)findViewById(R.id.filterBtn);
        filterBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent intent = new Intent();
                intent.setClass(ListCompetitionsActivity.this, FilterOptionsActivity.class);
                intent.putExtra(FilterOptionsActivity.FILTER_ID, filter != null ? filter : new CompetitionsFilter());
                startActivityForResult(intent, 1);
                fabButtonMenu.collapse();
            }
        });
    }

    /**
     * Funcion encargada de obtener el listado de competiciones a mostrar en pantalla. Verifica que exista acceso a internet. De no ser asi, el listado se
     * obtiene de las preferencias almacenadas previamente.
     */
    private void getCompetitionsList() {
        String competitionsJson = PreferenceManager.getDefaultSharedPreferences(getApplicationContext()).getString(JSON_COMPETICIONES_ID + "_" + idDisciplina , "");
        if(!Utils.isNetworkStatusAvialable(getApplicationContext())) {
            boolean checkStoredCompetitions = false;
            if(competitionsJson != null && !competitionsJson.equals("")){
                checkStoredCompetitions = true;
                ArrayList<Competition> competitionsList = jsonParser.jsonToCompeticiones(competitionsJson);
                this.completeCompetitionsList = competitionsList;
                updateComponents(null);
            }
            DialogCreator.createAlertConnectionDialog(ListCompetitionsActivity.this, checkStoredCompetitions).show();
        }else{
            new CompeticionesAsyncTask().execute("http://diplomaandroidupv-jovisal1.rhcloud.com/rest/eventos/get?idDisciplina=" + idDisciplina);
        }
    }

    /**
     * Inicializa todos los componentes presentes en el modo de visualizacion de calendario.
     *
     */
    public void initCalendar() {
        calendarCompetitionsView = (LinearLayout) findViewById(R.id.calendarCompetitionsView);
        caldroidFragment = new CaldroidFragment();
        Bundle args = new Bundle();
        Calendar cal = Calendar.getInstance();
        args.putInt(CaldroidFragment.MONTH, cal.get(Calendar.MONTH) + 1);
        args.putInt(CaldroidFragment.YEAR, cal.get(Calendar.YEAR));
        args.putBoolean(CaldroidFragment.ENABLE_SWIPE, true);
        args.putBoolean(CaldroidFragment.SIX_WEEKS_IN_CALENDAR, true);
        args.putInt(CaldroidFragment.START_DAY_OF_WEEK, CaldroidFragment.MONDAY);
        args.putInt(CaldroidFragment.THEME_RESOURCE, R.style.CaldroidDefaultCompeticionesDeportivas);
        caldroidFragment.setArguments(args);
        FragmentTransaction t = getSupportFragmentManager().beginTransaction();
        t.replace(R.id.calendar, caldroidFragment);
        t.commit();
        final CaldroidListener listener = new CaldroidListener() {

            @Override
            public void onSelectDate(Date date, View view) {
                setCompetitionsListForDate(date);
            }
            @Override
            public void onChangeMonth(int month, int year) {}
            @Override
            public void onLongClickDate(Date date, View view) {}
            @Override
            public void onCaldroidViewCreated() { }

        };
        caldroidFragment.setCaldroidListener(listener);
    }

    /**
     * Funcion que obtiene el listado de competiciones que se celebran en una fecha determinada. Posteriormente este se muestra
     * al seleccionar un dia concretro en el calendario
     *
     * @param date
     */
    private void setCompetitionsListForDate(Date date) {
        StringBuilder listadoCompeticiones = new StringBuilder();
        for (int i = 0; i < filteredCompetitionsList.size(); i++) {
            Competition selectedCompetition = filteredCompetitionsList.get(i);
            Calendar cal = Calendar.getInstance();
            cal.setTime(selectedCompetition.getFecha());
            cal.set(Calendar.HOUR_OF_DAY, 0);
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.SECOND, 0);
            cal.set(Calendar.MILLISECOND, 0);
            if(cal.getTime().compareTo(date) == 0){
                listadoCompeticiones.append("-" + selectedCompetition.getTitulo() + "\n");
            }
        }

        String competitionsListString = listadoCompeticiones.toString();
        if(!competitionsListString.equals("")){
            DialogCreator.createDialogCompetitionsList(ListCompetitionsActivity.this, competitionsListString, date).show();
        }
    }

    /**
     * Inicializa el componente FAB de pantalla
     */
    private void initFabButton() {
        fabButtonMenu = (FloatingActionsMenu) findViewById(R.id.fabButtonMenu);
        mapBtn = (FloatingActionButton) findViewById(R.id.mapBtn);
        calendarBtn = (FloatingActionButton) findViewById(R.id.calendarBtn);
        listBtn = (FloatingActionButton) findViewById(R.id.listBtn);
        listBtn.setVisibility(View.GONE);

        mapBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mapsCompetitionsView.setVisibility(View.VISIBLE);
                calendarCompetitionsView.setVisibility(View.INVISIBLE);
                recyclerCompetitionsView.setVisibility(View.INVISIBLE);
                mapBtn.setVisibility(View.GONE);
                calendarBtn.setVisibility(View.VISIBLE);
                listBtn.setVisibility(View.VISIBLE);
                fabButtonMenu.collapse();
                getSupportActionBar().hide();
            }
        });

        calendarBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mapsCompetitionsView.setVisibility(View.INVISIBLE);
                calendarCompetitionsView.setVisibility(View.VISIBLE);
                recyclerCompetitionsView.setVisibility(View.INVISIBLE);
                mapBtn.setVisibility(View.VISIBLE);
                calendarBtn.setVisibility(View.GONE);
                listBtn.setVisibility(View.VISIBLE);
                fabButtonMenu.collapse();
                getSupportActionBar().hide();
            }
        });

        listBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mapsCompetitionsView.setVisibility(View.INVISIBLE);
                calendarCompetitionsView.setVisibility(View.INVISIBLE);
                recyclerCompetitionsView.setVisibility(View.VISIBLE);
                mapBtn.setVisibility(View.VISIBLE);
                calendarBtn.setVisibility(View.VISIBLE);
                listBtn.setVisibility(View.GONE);
                fabButtonMenu.collapse();
                getSupportActionBar().show();
            }
        });
    }

    /**
     * Inicializa el componente RecyclerView
     */
    private void initRecyclerView() {
        recyclerCompetitionsView = (RecyclerView) findViewById(R.id.recyclerCompetitionsView);
        recyclerCompetitionsView.setLayoutManager(new LinearLayoutManager(this));
        recyclerCompetitionsView.addOnScrollListener(new HidingScrollListener() {
            @Override
            public void onHide() {
                FrameLayout.LayoutParams lp = (FrameLayout.LayoutParams) fabButtonMenu.getLayoutParams();
                int fabBottomMargin = lp.bottomMargin;
                fabButtonMenu.animate().translationY(fabButtonMenu.getHeight() + fabBottomMargin).setInterpolator(new AccelerateInterpolator(2)).start();
            }

            @Override
            public void onShow() {
                fabButtonMenu.animate().translationY(0).setInterpolator(new DecelerateInterpolator(2)).start();
            }
        });
    }

    /**
     * Funcion encargada de actualizar los distintos componentes de pantalla con el nuevo listado de competiciones obtenido
     *
     * @param filter
     */
    private void updateComponents(CompetitionsFilter filter) {
        if(filter == null ||  this.filteredCompetitionsList == null){
            this.filteredCompetitionsList = this.completeCompetitionsList;
        }else {
            this.filteredCompetitionsList = filter.applyFilter(this.completeCompetitionsList, getApplicationContext());
            competitionsMap.clear();
            caldroidFragment.clearBackgroundResourceForDates(lastSelectedDates);
            caldroidFragment.refreshView();
            lastSelectedDates = new ArrayList<Date>();
        }

        for (int i = 0; i < filteredCompetitionsList.size(); i++) {
            Competition competitionSel = filteredCompetitionsList.get(i);
            lastSelectedDates.add(competitionSel.getFecha());
            caldroidFragment.setTextColorForDate(R.color.color_white, competitionSel.getFecha());
            caldroidFragment.setBackgroundResourceForDate(R.color.color_primary, competitionSel.getFecha());
            String[] competitionCoordinates = competitionSel.getLocalidadGmaps().split(",");
            LatLng position = new LatLng(new Double(competitionCoordinates[0]), new Double(competitionCoordinates[1]));
            competitionsMap.addMarker(new MarkerOptions().position(position).title(competitionSel.getTitulo()));
        }

        caldroidFragment.refreshView();
        RecyclerAdapterCompetition recyclerAdapter = new RecyclerAdapterCompetition(this.filteredCompetitionsList);
        recyclerCompetitionsView.swapAdapter(recyclerAdapter, false);
    }

    /**
     * AsyncTask encargada de obtener el listado de competiciones a mostrar.
     */
    private class CompeticionesAsyncTask extends AsyncTask<String, Integer, String>{
        private ProgressBar progressDialog = (ProgressBar) findViewById(R.id.marker_progress);

        @Override
        protected void onPreExecute() {
            progressDialog.setVisibility(View.VISIBLE);
        }

        @Override
        protected String doInBackground(String... urls) {
            publishProgress(25);
            String jsonCompeticiones = jsonParser.getJSONFromUrl(urls[0]);
            publishProgress(50);
            return jsonCompeticiones;
        }

        @Override
        protected void onProgressUpdate(Integer... values) {
            super.onProgressUpdate(values);
        }

        @Override
        protected void onPostExecute(String jsonCompeticiones) {
            super.onPostExecute(jsonCompeticiones);
            PreferenceManager.getDefaultSharedPreferences(getApplicationContext()).edit().putString(JSON_COMPETICIONES_ID + "_" + idDisciplina, jsonCompeticiones).commit();
            ArrayList<Competition> competitionsList = jsonParser.jsonToCompeticiones(jsonCompeticiones);
            completeCompetitionsList = competitionsList;
            updateComponents(null);
            progressDialog.setVisibility(View.INVISIBLE);
        }
    }


    @Override
    public boolean onCreateOptionsMenu(final Menu menu) {
        // Inflate the options menu from XML
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_competitions_list, menu);
        final MenuItem menuSearchItem = menu.findItem(R.id.action_search);

        // Get the SearchView and set the searchable configuration
        SearchManager searchManager = (SearchManager) getSystemService(Context.SEARCH_SERVICE);
        SearchView searchView = (SearchView) menuSearchItem.getActionView();

        // Assumes current activity is the searchable activity
        searchView.setSearchableInfo(searchManager.getSearchableInfo(getComponentName()));
        searchView.setIconifiedByDefault(false); // Do not iconify the widget; expand it by default
        searchView.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextSubmit(String s) {
                InputMethodManager inputManager = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if (getCurrentFocus() != null && getCurrentFocus().getWindowToken() != null) {
                    inputManager.hideSoftInputFromWindow(getCurrentFocus().getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
                }
                menuSearchItem.collapseActionView();
                return false;
            }

            @Override
            public boolean onQueryTextChange(String searchString) {
                if (searchString.equals("")) {
                    if(filter == null) filter = new CompetitionsFilter();
                    filter.setFilterText(searchString);
                    updateComponents(filter);
                }
                return false;
            }
        });

        return true;
    }

    @Override
    protected void onNewIntent(Intent intent) {
        if(intent.getAction() != null){
            if(intent.getAction().equals(Intent.ACTION_SEARCH)){
                if(filter == null) filter = new CompetitionsFilter();
                filter.setFilterText(intent.getStringExtra(SearchManager.QUERY));
                updateComponents(filter);
            }
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent intent) {
        super.onActivityResult(requestCode, resultCode, intent);
        if(resultCode == FilterOptionsActivity.FILTER_APPLIED ) {
            filter = (CompetitionsFilter) intent.getSerializableExtra(FilterOptionsActivity.FILTER_ID);
            updateComponents(filter);
        }
    }
}
