package com.diplomaAndroidUpv.org.competicionesDeportivas.Activity;

import android.content.Intent;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageButton;
import android.widget.Toast;

import com.diplomaAndroidUpv.org.competicionesDeportivas.Listener.RecyclerItemClickListener;
import com.diplomaAndroidUpv.org.competicionesDeportivas.R;
import com.diplomaAndroidUpv.org.competicionesDeportivas.RecyclerView.Categories.RecyclerAdapter;
import com.diplomaAndroidUpv.org.competicionesDeportivas.RecyclerView.Categories.RecyclerAdapterItem;
import com.diplomaAndroidUpv.org.competicionesDeportivas.RecyclerView.Competitions.RecyclerAdapterCompetition;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Set;


/**
 * Activity encarga de de mostrar la pantalla principal de la aplicacion
 */
public class MainActivity extends AppCompatActivity {
    private ActionBar actionBar;
    private ImageButton settingsButton;
    private RecyclerView recyclerView;
    private List<RecyclerAdapterItem> itemList, selectedItemList;
    private Comparator<RecyclerAdapterItem> recyclerAdapterComparator = new Comparator<RecyclerAdapterItem>() {
        @Override
        public int compare(RecyclerAdapterItem recyclerAdapter1, RecyclerAdapterItem recyclerAdapter2) {
            return recyclerAdapter1.getIdDisciplina().compareTo(recyclerAdapter2.getIdDisciplina()) ;
        }
    };


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        initRecyclerView();
        createItemList();
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_main, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_settings:
                Intent i = new Intent(MainActivity.this, SettingsActivity.class);
                startActivity(i);
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        createItemList();
    }

    @Override
    protected void onRestart() {
        super.onRestart();
        createItemList();
    }

    /**
     * Inicializa el RecyclerView de la pantalla principal
     */
    private void initRecyclerView() {
        recyclerView = (RecyclerView) findViewById(R.id.recyclerView);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));
        recyclerView.addOnItemTouchListener(
                new RecyclerItemClickListener(getApplicationContext(), new RecyclerItemClickListener.OnItemClickListener() {
                    @Override
                    public void onItemClick(View view, int position) {
                        Intent i = new Intent(MainActivity.this, ListCompetitionsActivity.class);
                        RecyclerAdapterItem selectedCompetition = itemList.get(position);
                        i.putExtra("idDisciplina", selectedCompetition.getIdDisciplina());
                        startActivity(i);
                    }
                })
        );
    }

    /**
     * Crea el listado a mostrar de tipos de competición
     */
    private void createItemList() {
        itemList = new ArrayList<RecyclerAdapterItem>();
        Set<String> selections =  PreferenceManager.getDefaultSharedPreferences(MainActivity.this).getStringSet("competiciones_visibles",null);
        Iterator iterator = selections.iterator();
        while(iterator.hasNext()){
            String element = (String) iterator.next();
            if(element.equals("1")){
                itemList.add(new RecyclerAdapterItem(getString(R.string.running), R.drawable.running, new Long(1)));
            }
            if(element.equals("2")){
                itemList.add(new RecyclerAdapterItem(getString(R.string.trail), R.drawable.trail, new Long(2)));
            }
            if(element.equals("3")){
                itemList.add(new RecyclerAdapterItem(getString(R.string.triatlon), R.drawable.triatlon_foto, new Long(3)));
            }
            if(element.equals("4")){
                itemList.add(new RecyclerAdapterItem(getString(R.string.ciclismo), R.drawable.ciclismo, new Long(4)));
            }
        }

        Collections.sort(itemList,recyclerAdapterComparator);
        RecyclerAdapter recyclerAdapter = new RecyclerAdapter(itemList);
        recyclerView.swapAdapter(recyclerAdapter,false);
    }


}