package com.diplomaAndroidUpv.org.competicionesDeportivas.Fragment;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.preference.CheckBoxPreference;
import android.preference.EditTextPreference;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceFragment;
import android.preference.PreferenceManager;
import android.provider.Settings;
import android.text.Html;
import android.util.Log;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.diplomaAndroidUpv.org.competicionesDeportivas.Activity.MainActivity;
import com.diplomaAndroidUpv.org.competicionesDeportivas.Activity.SettingsActivity;
import com.diplomaAndroidUpv.org.competicionesDeportivas.R;
import com.diplomaAndroidUpv.org.competicionesDeportivas.Util.DialogCreator;
import com.diplomaAndroidUpv.org.competicionesDeportivas.Util.Utils;

public class SettingsFragment extends PreferenceFragment implements Preference.OnPreferenceChangeListener, LocationListener {

    private CheckBoxPreference positionPref;
    SharedPreferences prefs ;
    private LocationManager locationManager;
    private String provider;
    private static final long MIN_TIME = 1000;
    private static final long MIN_DISTANCE = 1;
    private boolean posicionRegistrada = false;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        addPreferencesFromResource(R.xml.preferences);
        this.prefs = getPreferenceManager().getSharedPreferences();
        this.posicionRegistrada = false;
        initLocationManager();
        initPositionPreferences();
    }

    @Override
    public void onResume() {
        super.onResume();
        posicionRegistrada = false;
    }

    @Override
    public void onPause() {
        super.onPause();
        if(locationManager != null) {
            locationManager.removeUpdates(this);
        }
        posicionRegistrada = false;
    }

    /**
     * Inicializa elementos de posicionamiento geográfico necesarios para el filtrado por posición
     */
    private void initLocationManager() {
        locationManager = (LocationManager)getActivity().getSystemService(Context.LOCATION_SERVICE);
        locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, MIN_TIME, MIN_DISTANCE, this);
        Criteria locationCritera = new Criteria();
       /* locationCritera.setAccuracy(Criteria.ACCURACY_MEDIUM);
        locationCritera.setAltitudeRequired(false);
        locationCritera.setCostAllowed(false);
        provider = locationManager.getBestProvider(locationCritera, true);*/
    }

    /**
     * Obtiene la posición actual
     */
    private void getCurrentPosition() {
        Location location = null;
        if(location != null){
            location = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
            PreferenceManager.getDefaultSharedPreferences(getActivity().getApplicationContext()).edit().putLong("myLat", Double.doubleToLongBits(location.getLatitude())).commit();
            PreferenceManager.getDefaultSharedPreferences(getActivity().getApplicationContext()).edit().putLong("myLong",Double.doubleToLongBits(location.getLongitude())).commit();
            savePosition(location.getLatitude(), location.getLongitude());
        }

    }

    /**
     * Inicializa las preferencias referentes a la posicion
     */
    private void initPositionPreferences() {
        this.positionPref = (CheckBoxPreference) findPreference("position");
        this.positionPref.setOnPreferenceChangeListener(this);
        if(positionPref.isChecked()){
            double latitude = Double.longBitsToDouble(PreferenceManager.getDefaultSharedPreferences(getActivity().getApplicationContext()).getLong("myLat", 0));
            double longitude = Double.longBitsToDouble(PreferenceManager.getDefaultSharedPreferences(getActivity().getApplicationContext()).getLong("myLong", 0));
            savePosition(latitude,longitude);
        }else{
            String positionPrefSummary = this.positionPref.getSummary().toString();
            if(positionPrefSummary.indexOf("(") != -1) {
                positionPrefSummary = positionPrefSummary.substring(0, positionPrefSummary.indexOf("("));
            }
            this.positionPref.setSummary(positionPrefSummary);
        }
    }

    @Override
    public boolean onPreferenceChange(Preference preference, Object o) {
        if(preference.getKey().equals("position")) {
            if(!prefs.getBoolean("position", true)){
                if (!locationManager.isProviderEnabled( LocationManager.GPS_PROVIDER )){
                    DialogCreator.createDialogGPSActivation(getActivity()).show();
                    return false;
                }
                getCurrentPosition();
            }else{
                removeSavedPosition();
            }
        }
        return true;
    }

    /**
     * Elimina cualquier tipo de informacion relativa a la posicion geografica almacenada
     */
    private void removeSavedPosition() {
        PreferenceManager.getDefaultSharedPreferences(getActivity().getApplicationContext()).edit().remove("locationString").commit();
        PreferenceManager.getDefaultSharedPreferences(getActivity().getApplicationContext()).edit().remove("myLat").commit();
        PreferenceManager.getDefaultSharedPreferences(getActivity().getApplicationContext()).edit().remove("myLong").commit();
        String positionPrefSummary = this.positionPref.getSummary().toString();
        if(positionPrefSummary.indexOf("(") != -1) {
            positionPrefSummary = positionPrefSummary.substring(0, positionPrefSummary.indexOf("("));
        }
        this.positionPref.setSummary(positionPrefSummary);
    }

    /**
     * Almacena la posicion actual y la muestra en las preferencias
     * @param latitude
     * @param longitude
     */
    public void savePosition(double latitude, double longitude){
        String locationString = Utils.getAddressFromLocation(latitude, longitude,getActivity().getApplicationContext());
        PreferenceManager.getDefaultSharedPreferences(getActivity().getApplicationContext()).edit().putString("locationString", locationString).commit();
        String positionPrefSummary = this.positionPref.getSummary().toString();
        if(positionPrefSummary.indexOf("(") != -1) {
            positionPrefSummary = positionPrefSummary.substring(0, positionPrefSummary.indexOf("("));
        }
        if(locationString != null && !locationString.equals("")) {
            this.positionPref.setSummary(positionPrefSummary + " " + Html.fromHtml(" \n\n ( <b>" + locationString + "</b>)"));
        }
    }

    @Override
    public void onLocationChanged(Location location) {
        if(!posicionRegistrada && location != null){
            PreferenceManager.getDefaultSharedPreferences(getActivity().getApplicationContext()).edit().putLong("myLat", Double.doubleToLongBits(location.getLatitude())).commit();
            PreferenceManager.getDefaultSharedPreferences(getActivity().getApplicationContext()).edit().putLong("myLong",Double.doubleToLongBits(location.getLongitude())).commit();
            savePosition(location.getLatitude(), location.getLongitude());
        }
        posicionRegistrada = true;
    }

    @Override
    public void onStatusChanged(String s, int i, Bundle bundle) {

    }

    @Override
    public void onProviderEnabled(String s) {

    }

    @Override
    public void onProviderDisabled(String s) {

    }
}