package com.diplomaAndroidUpv.org.competicionesDeportivas.Util;


import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.diplomaAndroidUpv.org.competicionesDeportivas.Activity.FilterOptionsActivity;
import com.diplomaAndroidUpv.org.competicionesDeportivas.Entity.Competition;
import com.diplomaAndroidUpv.org.competicionesDeportivas.R;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Clase encargada de generar los distintos cuadros de dialogo que se muestran en la aplicacion
 */
public class DialogCreator {


    /**
     * Dialogo de alerta por falta de conexion a internet
     * @param originActivity
     * @param checkStoredCompetitions
     * @return
     */
    public static AlertDialog createAlertConnectionDialog(final Activity originActivity, final boolean checkStoredCompetitions) {
        AlertDialog.Builder alertDialog = new AlertDialog.Builder(originActivity);
        alertDialog.setMessage(checkStoredCompetitions ? R.string.listado_carreras_no_actualizado : R.string.no_conexion_internet);
        alertDialog.setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                if (!checkStoredCompetitions) {
                    originActivity.finish();
                }
            }
        });
        AlertDialog alert = alertDialog.create();
        return alert;
    }

    /**
     * Crea el dialogo que muestra el listado de carreras disponible un dia determinado
     * @param originActivity
     * @param competitionsListString
     * @param competitionDate
     * @return
     */
    public static AlertDialog createDialogCompetitionsList(final Activity originActivity, String competitionsListString, Date competitionDate){
        SimpleDateFormat formatter = new SimpleDateFormat("dd/MM/yyyy");
        LayoutInflater inflater = LayoutInflater.from(originActivity);
        View view = inflater.inflate(R.layout.dialog_list_competitions, null);
        TextView textview=(TextView)view.findViewById(R.id.textmsg);
        textview.setText(competitionsListString);
        AlertDialog.Builder alertDialog = new AlertDialog.Builder(originActivity);
        alertDialog.setTitle(originActivity.getResources().getString(R.string.competiciones_del) + " " + formatter.format(competitionDate));
        alertDialog.setView(view);
        alertDialog.setPositiveButton(android.R.string.yes,null);
        AlertDialog alert = alertDialog.create();
        return alert;
    }

    /**
     * Crea el dialogo que alerta al usuario de que es necesario que tenga activo el posicionamiento en su dispositivo
     * @param originActivity
     * @return
     */
    public static AlertDialog createDialogGPSActivation(final Activity originActivity){
        final AlertDialog.Builder builder = new AlertDialog.Builder(originActivity);
        builder.setMessage(originActivity.getResources().getString(R.string.please_turn_on_gps))
                .setCancelable(false)
                .setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
                    public void onClick(final DialogInterface dialog, final int id) {
                        dialog.cancel();
                    }
                });
        final AlertDialog alert = builder.create();
        return alert;
    }

    /**
     * Crea el dialogo que permite al usuario 'agendar' una competición a su calendario de google
     * @param originActivity
     * @param competicion
     * @return
     */
    public static AlertDialog createDialogCompetitionSelection(final Activity originActivity, final Competition competicion){
        AlertDialog.Builder alertDialog = new AlertDialog.Builder(originActivity);
        alertDialog.setMessage(R.string.desea_guardar_carrera_calendario);
        alertDialog.setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                Utils.addCompetitionToCalendar(originActivity, competicion);
            }
        });
        alertDialog.setNegativeButton(android.R.string.no, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                dialog.cancel();
            }
        });
        AlertDialog alert = alertDialog.create();
        return alert;
    }

    /**
     * Crea el dialogo que muestra las distintas opciones cuando se selecciona una competición del listado.
     * @param context
     * @param competition
     * @return
     */
    public static AlertDialog createDialogCompetitionOptions(final Context context, final Competition competition) {
        LayoutInflater inflater = LayoutInflater.from(context);
        View view = inflater.inflate(R.layout.dialog_competition_options, null);
        ImageButton infoBtn = (ImageButton)view.findViewById(R.id.infoBtn);
        ImageButton calendarBtn = (ImageButton)view.findViewById(R.id.calendarBtn);
        TextView competitionTitle = (TextView)view.findViewById(R.id.competitionTitle);
        LinearLayout infoOptionsLayout = (LinearLayout)view.findViewById(R.id.infoOptionsLayout);
        if(competition.getUrlEvento() != null && !competition.getUrlEvento().equals("")){
            infoOptionsLayout.setVisibility(View.VISIBLE);
        }else{
            infoOptionsLayout.setVisibility(View.GONE);
        }

        infoBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                String url = competition.getUrlEvento();
                Intent i = new Intent(Intent.ACTION_VIEW);
                i.setData(Uri.parse(url));
                context.startActivity(i);
            }
        });

        calendarBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Utils.addCompetitionToCalendar(context,competition);
            }
        });

        competitionTitle.setText(competition.getTitulo());
        AlertDialog.Builder alertDialog = new AlertDialog.Builder(context);
        alertDialog.setView(view);
        alertDialog.setPositiveButton(android.R.string.yes, null);
        AlertDialog alert = alertDialog.create();
        return alert;
    }

    public static AlertDialog createDialogSavePositionActivation(final Activity originActivity) {
        final AlertDialog.Builder builder = new AlertDialog.Builder(originActivity);
        builder.setMessage(originActivity.getResources().getString(R.string.please_set_your_position))
                .setCancelable(false)
                .setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
                    public void onClick(final DialogInterface dialog, final int id) {
                        dialog.cancel();
                    }
                });
        final AlertDialog alert = builder.create();
        return alert;
    }
}
