package com.diplomaAndroidUpv.org.competicionesDeportivas.Util;


import android.util.Log;
import com.diplomaAndroidUpv.org.competicionesDeportivas.Entity.Competition;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonParseException;
import com.google.gson.stream.JsonReader;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.lang.reflect.Type;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;

/**
 * Clase que contiene lo necesario para realizar peticiones a la parte servidora y transformar el json recibido en
 * listados de objetos
 */
public class JsonParser {

    /**
     * Obtiene el json fruto de realizar una petición a una url determinada
     *
     * @param urlString
     * @return
     */
    public String getJSONFromUrl(String urlString) {
        StringBuilder sb = new StringBuilder();

        try {
            URL url = new URL(urlString);
            HttpURLConnection conexion = (HttpURLConnection) url.openConnection();
            if(conexion.getResponseCode() == HttpURLConnection.HTTP_OK){
                BufferedReader reader = new BufferedReader(new InputStreamReader(conexion.getInputStream()));
                String linea = reader.readLine();
                while(linea != null && !linea.equals("")){
                    sb.append(linea + "n");
                    linea = reader.readLine();
                }
                reader.close();
            }
        } catch (IOException e) {
            Log.e("Buffer Error", "Error converting result " + e.toString());
        }

        return sb.toString();
    }

    /**
     * Transforma un json en un listado de objetos de tipo Competition
     * @param json
     * @return
     */
    public ArrayList<Competition> jsonToCompeticiones(String json){
       GsonBuilder builder = new GsonBuilder();
       builder.registerTypeAdapter(Date.class, new JsonDeserializer<Date>() {
            public Date deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context) throws JsonParseException {
                return new Date(json.getAsJsonPrimitive().getAsLong());
            }
        });
        Gson gson = builder.create();
        JsonReader reader = new JsonReader(new StringReader(json));
        reader.setLenient(true);

        Competition[] competiciones = gson.fromJson(reader, Competition[].class);
        if(competiciones != null) {
            return new ArrayList<Competition>(Arrays.asList(competiciones));
        }
        return null;
    }
}
